package com.winhc.bigdata.spark.ng.credit_punishment

import com.winhc.bigdata.spark.ng.credit_punishment.udf.CreditPunishmentCaseAggUDF
import com.winhc.bigdata.spark.ng.utils.StartAndEndDsUtils
import com.winhc.bigdata.spark.udf.BaseFunc
import com.winhc.bigdata.spark.utils.BaseUtil.isWindows
import com.winhc.bigdata.spark.utils.{BaseUtil, LoggingUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

import scala.annotation.meta.getter

/**
 * @author: XuJiakai
 * @date: 2021/5/10 18:23
 *
 *        根据案件聚合
 */
case class CreditPunishmentCaseAgg(s: SparkSession
                                  ) extends LoggingUtils with BaseFunc {
  @(transient@getter) val spark: SparkSession = s

  private val target_tab = CreditPunishment.caseTargetTab

  init()

  private def init(): Unit = {
    spark.udf.register("credit_punishment_case_agg", new CreditPunishmentCaseAggUDF)

    def generate_entity_id(keyno: String, card_num: String, person_id: String, credit_punishment_case_id: String): String = {

      if (StringUtils.isNotEmpty(keyno) && keyno.length == 32) {
        return keyno
      }
      if (BaseUtil.is_id_card(card_num)) {
        return person_id
      }
      credit_punishment_case_id
    }

    spark.udf.register("generate_entity_id", generate_entity_id _)


    cleanup()
    sql(
      s"""
         |CREATE TABLE IF NOT EXISTS $target_tab
         |(
         |    credit_punishment_case_id STRING COMMENT '信用惩戒查询案件id'
         |    ,credit_punishment_entity_id STRING COMMENT '信用惩戒对象主体id'
         |    ,name STRING COMMENT '惩戒主体名称'
         |    ,court_name STRING COMMENT '法院名称'
         |    ,case_no STRING COMMENT '案号'
         |    ,rowkey STRING COMMENT '涉案rowkey'
         |    ,card_num STRING COMMENT '案号'
         |    ,keyno STRING COMMENT '惩戒主体id'
         |    ,label STRING COMMENT '标签'
         |    ,case_create_time STRING COMMENT '立案时间'
         |    ,record_num bigint COMMENT '记录条数'
         |    ,total_exec_amount STRING COMMENT '累计被执行总金额'
         |    ,total_no_exec_amount STRING COMMENT '疑似当前欠款总金额'
         |    ,zxr_total_exec_amount STRING COMMENT '被执行人当前被执行总金额'
         |    ,final_case_exec_amount STRING COMMENT '终本案件执行标的总金额'
         |    ,final_case_no_exec_amount STRING COMMENT '终本案件未履行总金额'
         |    ,company_dishonest_info_num BIGINT COMMENT '失信人条数'
         |    ,company_zxr_num BIGINT COMMENT '被执条数'
         |    ,company_zxr_final_case_num BIGINT COMMENT '终本案件条数'
         |    ,company_zxr_restrict_num BIGINT COMMENT '限高条数'
         |    ,deleted BIGINT COMMENT '0：未删除，1：删除'
         |)
         |COMMENT '信用惩戒查询案件表'
         |PARTITIONED BY
         |(
         |    ds STRING COMMENT '分区'
         |)
         |""".stripMargin)

  }


  private def getCreditPunishmentDataExtraction(ds: String): String = {
    val tmp_view = "credit_punishment_data_extraction_calc"
    if (ds == null) {
      sql(
        s"""
           |SELECT  *
           |FROM    ${CreditPunishment.extractionTargetTab}
           |WHERE   ds > '0'
           |""".stripMargin)
        .createTempView(tmp_view)
    } else {
      sql(
        s"""
           |SELECT  *
           |FROM    ${CreditPunishment.extractionTargetTab}
           |WHERE   ds > '$ds'
           |""".stripMargin)
        .createTempView(tmp_view)
    }
    tmp_view
  }


  def calc(ds:String): Unit = {
    val args = StartAndEndDsUtils(spark).get_gt_ds(CreditPunishment.extractionTargetTab, CreditPunishment.caseTargetTab)

    val org_tab = getCreditPunishmentDataExtraction(args.inc_tab_gt_ds)
    sql(
      s"""
         |INSERT ${if (isWindows) "INTO" else "OVERWRITE"} TABLE $target_tab PARTITION(ds='${ds}')
         |SELECT  md5(cleanup(CONCAT_WS('',name,court_name,case_no))) AS credit_punishment_case_id
         |        ,generate_entity_id(mm['keyno'],mm['card_num'],md5(CONCAT_WS('',mm['card_num'],name)),md5(cleanup(CONCAT_WS('',name,court_name,case_no)))) as credit_punishment_entity_id
         |        ,name
         |        ,court_name
         |        ,case_no
         |        ,mm['rowkey'] AS rowkey
         |        ,mm['card_num'] AS card_num
         |        ,mm['keyno'] AS keyno
         |        ,mm['label'] AS label
         |        ,mm['case_create_time'] AS case_create_time
         |        ,cast(mm['record_num'] as bigint) AS record_num
         |        ,mm['total_exec_amount'] AS total_exec_amount
         |        ,mm['total_no_exec_amount'] AS total_no_exec_amount
         |        ,mm['zxr_total_exec_amount'] AS zxr_total_exec_amount
         |        ,mm['final_case_exec_amount'] AS final_case_exec_amount
         |        ,mm['final_case_no_exec_amount'] AS final_case_no_exec_amount
         |        ,CAST(mm['company_dishonest_info_num'] AS BIGINT) AS company_dishonest_info_num
         |        ,CAST(mm['company_zxr_num'] AS BIGINT) AS company_zxr_num
         |        ,CAST(mm['company_zxr_final_case_num'] AS BIGINT) AS company_zxr_final_case_num
         |        ,CAST(mm['company_zxr_restrict_num'] AS BIGINT) AS company_zxr_restrict_num
         |        ,cast(mm['deleted'] as bigint) AS deleted
         |FROM    (
         |            SELECT  name
         |                    ,court_name
         |                    ,case_no
         |                    ,credit_punishment_case_agg(rowkey,tn,keyno,card_num,case_create_time,deleted,detail_data) AS mm
         |            FROM    $org_tab
         |            GROUP BY name
         |                     ,court_name
         |                     ,case_no
         |        )
         |WHERE   name IS NOT NULL and trim(name) <> ''
         |""".stripMargin)
  }
}
